#include <cmath>
using namespace std;

#include "josephus-graphics.h"
#include "gobjects.h"
#include "strlib.h"

static const double kWindowDimension = 10 * 72; // dimension in inches
static const double kPi = 3.14159;
static const double kPersonRadius = 15;

static const string kTitle = "Josephus Execution Simulation";
static const string kFont = "*-Bold-18";

static const string kDarkGray = "#444444";
static const string kLightBlue = "#aaaaff";
static const string kLightGreen = "#aaffaa";
static const string kLightRed = "#ffaaaa";

static const double kDelay = 30;

JosephusWindow::JosephusWindow() : GWindow(kWindowDimension, kWindowDimension) {
    numPeople = 0;
    // setWindowTitle(kTitle);
    // setVisible(true);
    setExitOnClose(true);
}

JosephusWindow::~JosephusWindow() {
    close();
}

void JosephusWindow::clear() {
    GRect *background = new GRect(0, 0, kWindowDimension, kWindowDimension);
    background->setColor("White");
    background->setFilled(true);
    background->setFillColor("White");
    add(background);
}

void JosephusWindow::setNumPeople(int numPeople) {
    if (numPeople <= 0)
        error("JosephusWindow::setNumPeople can only handle a positive integer argument.");

    this->numPeople = numPeople;
    drawCircle();
}

void JosephusWindow::resurrect(int person) {
	drawPerson(person, kLightBlue, true);
}

void JosephusWindow::spare(int person) {
	drawPerson(person, kLightGreen, true);
}

void JosephusWindow::execute(int person) {
    drawPerson(person, kLightRed, false); // very faint
}

void JosephusWindow::drawCircle() {
    clear();
    for (int person = 1; person <= numPeople; person++) {
        resurrect(person);
    }
}

void JosephusWindow::drawPerson(int person, string color, bool drawNumber) {
	double angleInDegrees = 90.0 - 360.0 * (person - 1) / numPeople;
	double angleInRadians = 2 * kPi * angleInDegrees / 360.0;
	double cx = kWindowDimension/2 + kWindowDimension * cos(angleInRadians) / 3;
	double cy = kWindowDimension/2 - kWindowDimension * sin(angleInRadians) / 3;
    
    GOval *circle = new GOval(cx - kPersonRadius, cy - kPersonRadius, 2 * kPersonRadius, 2 * kPersonRadius);
    circle->setFilled(true);
    circle->setColor(color);
    circle->setFillColor(color);
    add(circle);
	if (!drawNumber) return;
	string numString = integerToString(person);
    GText *label = new GText(numString);
    label->setColor(kDarkGray);
    label->setFont(kFont);
    add(label, cx - label->getWidth()/2, cy + 2 * label->getFontAscent()/5);
	::pause(kDelay);
}
